@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,6 +66,8 @@
 #define MIN_WINDOW_WIDTH 10
 #define MIN_WINDOW_HEIGHT 10
 
+#define KEYFILE_GROUP_REMMINA "remmina"
+
 static struct timespec times[2];
 
 static RemminaFile *
@@ -233,6 +235,94 @@ const RemminaProtocolSetting *find_protocol_setting(const gchar *name, RemminaPr
 	return NULL;
 }
 
+
+static void upgrade_sshkeys_202001_mig_common_setting(RemminaFile *remminafile, gboolean protocol_is_ssh, gboolean ssh_enabled, gchar *suffix)
+{
+	gchar *src_key;
+	gchar *dst_key;
+	const gchar *val;
+
+	src_key = g_strdup_printf("ssh_%s", suffix);
+	dst_key = g_strdup_printf("ssh_tunnel_%s", suffix);
+
+	val = remmina_file_get_string(remminafile, src_key);
+	if (!val) {
+		g_free(dst_key);
+		g_free(src_key);
+		return;
+	}
+
+	if (ssh_enabled && val && val[0] != 0)
+		remmina_file_set_string(remminafile, dst_key, val);
+
+	if (!protocol_is_ssh)
+		remmina_file_set_string(remminafile, src_key, NULL);
+
+	g_free(dst_key);
+	g_free(src_key);
+}
+
+static void upgrade_sshkeys_202001(RemminaFile *remminafile)
+{
+	TRACE_CALL(__func__);
+
+	gboolean protocol_is_ssh;
+	gboolean ssh_enabled;
+	const gchar *val;
+
+	if (remmina_file_get_string(remminafile, "ssh_enabled")) {
+
+		/* Upgrade ssh params from remmina pre 1.4 */
+
+		ssh_enabled = remmina_file_get_int(remminafile, "ssh_enabled", 0);
+		val = remmina_file_get_string(remminafile, "protocol");
+		protocol_is_ssh = (strcmp(val, "SSH") == 0);
+
+		upgrade_sshkeys_202001_mig_common_setting(remminafile, protocol_is_ssh, ssh_enabled, "stricthostkeycheck");
+		upgrade_sshkeys_202001_mig_common_setting(remminafile, protocol_is_ssh, ssh_enabled, "kex_algorithms");
+		upgrade_sshkeys_202001_mig_common_setting(remminafile, protocol_is_ssh, ssh_enabled, "hostkeytypes");
+		upgrade_sshkeys_202001_mig_common_setting(remminafile, protocol_is_ssh, ssh_enabled, "ciphers");
+		upgrade_sshkeys_202001_mig_common_setting(remminafile, protocol_is_ssh, ssh_enabled, "proxycommand");
+		upgrade_sshkeys_202001_mig_common_setting(remminafile, protocol_is_ssh, ssh_enabled, "passphrase");
+		upgrade_sshkeys_202001_mig_common_setting(remminafile, protocol_is_ssh, ssh_enabled, "auth");
+		upgrade_sshkeys_202001_mig_common_setting(remminafile, protocol_is_ssh, ssh_enabled, "privatekey");
+
+		val = remmina_file_get_string(remminafile, "ssh_loopback");
+		if (val) {
+			remmina_file_set_string(remminafile, "ssh_tunnel_loopback", val);
+			remmina_file_set_string(remminafile, "ssh_loopback", NULL);
+		}
+
+		val = remmina_file_get_string(remminafile, "ssh_username");
+		if (val) {
+			remmina_file_set_string(remminafile, "ssh_tunnel_username", val);
+			if (protocol_is_ssh)
+				remmina_file_set_string(remminafile, "username", val);
+			remmina_file_set_string(remminafile, "ssh_username", NULL);
+		}
+
+		val = remmina_file_get_string(remminafile, "ssh_password");
+		if (val) {
+			remmina_file_set_string(remminafile, "ssh_tunnel_password", val);
+			if (protocol_is_ssh)
+				remmina_file_set_string(remminafile, "password", val);
+			remmina_file_set_string(remminafile, "ssh_password", NULL);
+		}
+
+		val = remmina_file_get_string(remminafile, "ssh_server");
+		if (val) {
+			remmina_file_set_string(remminafile, "ssh_tunnel_server", val);
+			remmina_file_set_string(remminafile, "ssh_server", NULL);
+		}
+
+		/* Real key removal will be done by remmina_file_save() */
+
+		remmina_file_set_int(remminafile, "ssh_tunnel_enabled", ssh_enabled);
+
+	}
+
+}
+
 RemminaFile *
 remmina_file_load(const gchar *filename)
 {
@@ -255,18 +345,19 @@ remmina_file_load(const gchar *filename)
 	if (g_file_test(filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_EXISTS)) {
 		if (!g_key_file_load_from_file(gkeyfile, filename, G_KEY_FILE_NONE, NULL)) {
 			g_key_file_free(gkeyfile);
-			g_debug("Unable to load remmina profile file %s: g_key_file_load_from_file() returned NULL.\n", filename);
+			REMMINA_DEBUG ("Unable to load remmina profile file %s: g_key_file_load_from_file() returned NULL.\n", filename);
 			return NULL;
 		}
 	}
 
-	if (g_key_file_has_key(gkeyfile, "remmina", "name", NULL)) {
+	if (g_key_file_has_key(gkeyfile, KEYFILE_GROUP_REMMINA, "name", NULL)) {
+
 		remminafile = remmina_file_new_empty();
 
 		protocol_plugin = NULL;
 
 		/* Identify the protocol plugin and get pointers to its RemminaProtocolSetting structs */
-		proto = g_key_file_get_string(gkeyfile, "remmina", "protocol", NULL);
+		proto = g_key_file_get_string(gkeyfile, KEYFILE_GROUP_REMMINA, "protocol", NULL);
 		if (proto) {
 			protocol_plugin = (RemminaProtocolPlugin *)remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL, proto);
 			g_free(proto);
@@ -276,12 +367,13 @@ remmina_file_load(const gchar *filename)
 		secret_service_available = secret_plugin && secret_plugin->is_service_available();
 
 		remminafile->filename = g_strdup(filename);
-		keys = g_key_file_get_keys(gkeyfile, "remmina", NULL, NULL);
+		keys = g_key_file_get_keys(gkeyfile, KEYFILE_GROUP_REMMINA, NULL, NULL);
 		if (keys) {
+
 			for (i = 0; keys[i]; i++) {
 				key = keys[i];
 				if (protocol_plugin && remmina_plugin_manager_is_encrypted_setting(protocol_plugin, key)) {
-					s = g_key_file_get_string(gkeyfile, "remmina", key, NULL);
+					s = g_key_file_get_string(gkeyfile, KEYFILE_GROUP_REMMINA, key, NULL);
 					if (g_strcmp0(s, ".") == 0) {
 						if (secret_service_available) {
 							sec = secret_plugin->get_password(remminafile, key);
@@ -299,7 +391,7 @@ remmina_file_load(const gchar *filename)
 				} else {
 					/* If we find "resolution", then we split it in two */
 					if (strcmp(key, "resolution") == 0) {
-						resolution_str = g_key_file_get_string(gkeyfile, "remmina", key, NULL);
+						resolution_str = g_key_file_get_string(gkeyfile, KEYFILE_GROUP_REMMINA, key, NULL);
 						if (remmina_public_split_resolution_string(resolution_str, &w, &h)) {
 							remmina_file_set_string_ref(remminafile, "resolution_width", g_strdup_printf("%i", w));
 							remmina_file_set_string_ref(remminafile, "resolution_height", g_strdup_printf("%i", h));
@@ -310,14 +402,16 @@ remmina_file_load(const gchar *filename)
 						g_free(resolution_str);
 					} else {
 						remmina_file_set_string_ref(remminafile, key,
-									    g_key_file_get_string(gkeyfile, "remmina", key, NULL));
+									    g_key_file_get_string(gkeyfile, KEYFILE_GROUP_REMMINA, key, NULL));
 					}
 				}
 			}
-			g_strfreev(keys);
+
+			upgrade_sshkeys_202001(remminafile);
+
 		}
 	} else {
-		g_debug("Unable to load remmina profile file %s: cannot find key name= in section remmina.\n", filename);
+		REMMINA_DEBUG ("Unable to load remmina profile file %s: cannot find key name= in section remmina.\n", filename);
 		remminafile = NULL;
 	}
 
@@ -397,6 +491,30 @@ remmina_file_get_secret(RemminaFile *remminafile, const gchar *setting)
 	return g_strdup(remmina_file_get_string(remminafile, setting));
 }
 
+gchar *remmina_file_format_properties(RemminaFile *remminafile, const gchar *setting)
+{
+	gchar *res = NULL;
+	GString *fmt_str;
+	GDateTime *now;
+	gchar *date_str = NULL;
+
+	fmt_str = g_string_new(setting);
+	remmina_utils_string_replace_all(fmt_str, "%h", remmina_file_get_string(remminafile, "server"));
+	remmina_utils_string_replace_all(fmt_str, "%t", remmina_file_get_string(remminafile, "ssh_tunnel_server"));
+	remmina_utils_string_replace_all(fmt_str, "%u", remmina_file_get_string(remminafile, "username"));
+	remmina_utils_string_replace_all(fmt_str, "%U", remmina_file_get_string(remminafile, "ssh_tunnel_username"));
+	remmina_utils_string_replace_all(fmt_str, "%p", remmina_file_get_string(remminafile, "name"));
+	remmina_utils_string_replace_all(fmt_str, "%g", remmina_file_get_string(remminafile, "group"));
+
+	now = g_date_time_new_now_local ();
+	date_str = g_date_time_format(now, "YYYY-MM-DDTHH-MM-SSZ");
+	remmina_utils_string_replace_all(fmt_str, "%d", date_str);
+	g_free(date_str);
+
+	res = g_string_free(fmt_str, FALSE);
+	return res;
+}
+
 void remmina_file_set_int(RemminaFile *remminafile, const gchar *setting, gint value)
 {
 	TRACE_CALL(__func__);
@@ -460,7 +578,7 @@ void remmina_file_save(RemminaFile *remminafile)
 	if ((gkeyfile = remmina_file_get_keyfile(remminafile)) == NULL)
 		return;
 
-	g_debug("Saving profile");
+	REMMINA_DEBUG ("Saving profile");
 	/* get disablepasswordstoring */
 	nopasswdsave = remmina_file_get_int(remminafile, "disablepasswordstoring", 0);
 	/* Identify the protocol plugin and get pointers to its RemminaProtocolSetting structs */
@@ -480,47 +598,53 @@ void remmina_file_save(RemminaFile *remminafile)
 		if (remmina_plugin_manager_is_encrypted_setting(protocol_plugin, key)) {
 			if (remminafile->filename && g_strcmp0(remminafile->filename, remmina_pref_file)) {
 				if (secret_service_available && nopasswdsave == 0) {
-					g_debug ("We have a secret and disablepasswordstoring=0");
+					REMMINA_DEBUG ("We have a secret and disablepasswordstoring=0");
 					if (value && value[0]) {
 						if (g_strcmp0(value, ".") != 0)
 							secret_plugin->store_password(remminafile, key, value);
-						g_key_file_set_string(gkeyfile, "remmina", key, ".");
+						g_key_file_set_string(gkeyfile, KEYFILE_GROUP_REMMINA, key, ".");
 					} else {
-						g_key_file_set_string(gkeyfile, "remmina", key, "");
+						g_key_file_set_string(gkeyfile, KEYFILE_GROUP_REMMINA, key, "");
 						secret_plugin->delete_password(remminafile, key);
 					}
 				} else {
-					g_debug ("We have a password and disablepasswordstoring=0");
+					REMMINA_DEBUG ("We have a password and disablepasswordstoring=0");
 					if (value && value[0] && nopasswdsave == 0) {
 						s = remmina_crypt_encrypt(value);
-						g_key_file_set_string(gkeyfile, "remmina", key, s);
+						g_key_file_set_string(gkeyfile, KEYFILE_GROUP_REMMINA, key, s);
 						g_free(s);
 					} else {
-						g_key_file_set_string(gkeyfile, "remmina", key, "");
+						g_key_file_set_string(gkeyfile, KEYFILE_GROUP_REMMINA, key, "");
 					}
 				}
 				if (secret_service_available && nopasswdsave == 1) {
 					if (value && value[0]) {
 						if (g_strcmp0(value, ".") != 0) {
-							g_debug ("Deleting the secret in the keyring as disablepasswordstoring=1");
+							REMMINA_DEBUG ("Deleting the secret in the keyring as disablepasswordstoring=1");
 							secret_plugin->delete_password(remminafile, key);
-							g_key_file_set_string(gkeyfile, "remmina", key, ".");
+							g_key_file_set_string(gkeyfile, KEYFILE_GROUP_REMMINA, key, ".");
 						}
 					}
 				}
 			}
 		} else {
-			g_key_file_set_string(gkeyfile, "remmina", key, value);
+			g_key_file_set_string(gkeyfile, KEYFILE_GROUP_REMMINA, key, value);
 		}
 	}
 
 	/* Avoid storing redundant and deprecated "resolution" field */
-	g_key_file_remove_key(gkeyfile, "remmina", "resolution", NULL);
+	g_key_file_remove_key(gkeyfile, KEYFILE_GROUP_REMMINA, "resolution", NULL);
+
+	/* Delete old pre-1.4 ssh keys */
+	g_key_file_remove_key(gkeyfile, KEYFILE_GROUP_REMMINA, "ssh_enabled", NULL);
+	g_key_file_remove_key(gkeyfile, KEYFILE_GROUP_REMMINA, "save_ssh_server", NULL);
+	g_key_file_remove_key(gkeyfile, KEYFILE_GROUP_REMMINA, "save_ssh_username", NULL);
 
 	/* Store gkeyfile to disk (password are already sent to keyring) */
 	content = g_key_file_to_data(gkeyfile, &length, NULL);
+
 	if (g_file_set_contents(remminafile->filename, content, length, &err)) {
-		g_debug("Profile saved");
+		REMMINA_DEBUG ("Profile saved");
 	} else {
 		g_warning("Remmina connection profile cannot be saved, with error %d (%s)", err->code, err->message);
 	}
@@ -578,9 +702,9 @@ remmina_file_get_icon_name(RemminaFile *remminafile)
 	plugin = (RemminaProtocolPlugin *)remmina_plugin_manager_get_plugin(REMMINA_PLUGIN_TYPE_PROTOCOL,
 									    remmina_file_get_string(remminafile, "protocol"));
 	if (!plugin)
-		return REMMINA_APP_ID;
+		return g_strconcat (REMMINA_APP_ID, "-symbolic", NULL);
 
-	return remmina_file_get_int(remminafile, "ssh_enabled", FALSE) ? plugin->icon_name_ssh : plugin->icon_name;
+	return remmina_file_get_int(remminafile, "ssh_tunnel_enabled", FALSE) ? plugin->icon_name_ssh : plugin->icon_name;
 }
 
 RemminaFile *
@@ -603,13 +727,13 @@ void remmina_file_delete(const gchar *filename)
 
 	remminafile = remmina_file_load(filename);
 	if (remminafile) {
-		remmina_file_unsave_password(remminafile);
+		remmina_file_unsave_passwords(remminafile);
 		remmina_file_free(remminafile);
 	}
 	g_unlink(filename);
 }
 
-void remmina_file_unsave_password(RemminaFile *remminafile)
+void remmina_file_unsave_passwords(RemminaFile *remminafile)
 {
 	/* Delete all saved secrets for this profile */
 
@@ -721,17 +845,17 @@ remmina_file_touch(RemminaFile *remminafile)
 
 	if ((r = stat(remminafile->filename, &st)) < 0) {
 		if (errno != ENOENT)
-			remmina_log_printf("stat %s:", remminafile->filename);
+			REMMINA_DEBUG("stat %s:", remminafile->filename);
 	} else if (!r) {
 		times[0] = st.st_atim;
 		times[1] = st.st_mtim;
 		if (utimensat(AT_FDCWD, remminafile->filename, times, 0) < 0)
-			remmina_log_printf("utimensat %s:", remminafile->filename);
+			REMMINA_DEBUG("utimensat %s:", remminafile->filename);
 		return;
 	}
 
 	if ((fd = open(remminafile->filename, O_CREAT | O_EXCL, 0644)) < 0)
-		remmina_log_printf("open %s:", remminafile->filename);
+		REMMINA_DEBUG("open %s:", remminafile->filename);
 	close(fd);
 
 	remmina_file_touch(remminafile);
