@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,13 +73,14 @@ static void remmina_pref_gen_secret(void)
 	TRACE_CALL(__func__);
 	guchar s[32];
 	gint i;
-	GTimeVal gtime;
+	GDateTime *gtime;
 	GKeyFile *gkeyfile;
 	gchar *content;
 	gsize length;
 
-	g_get_current_time(&gtime);
-	srand(gtime.tv_sec);
+	gtime = g_date_time_new_now_utc();
+	srand(g_date_time_to_unix(gtime));
+	g_date_time_unref(gtime);
 
 	for (i = 0; i < 32; i++) {
 		s[i] = (guchar)(rand() % 256);
@@ -638,6 +639,19 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.vte_shortcutkey_select_all = GDK_KEY_a;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "vte_shortcutkey_increase_font", NULL))
+		remmina_pref.vte_shortcutkey_increase_font = g_key_file_get_integer(gkeyfile, "remmina_pref", "vte_shortcutkey_increase_font",
+			NULL);
+	else
+		remmina_pref.vte_shortcutkey_increase_font = GDK_KEY_Page_Up;
+
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "vte_shortcutkey_decrease_font", NULL))
+		remmina_pref.vte_shortcutkey_decrease_font = g_key_file_get_integer(gkeyfile, "remmina_pref", "vte_shortcutkey_decrease_font",
+			NULL);
+	else
+		remmina_pref.vte_shortcutkey_decrease_font = GDK_KEY_Page_Down;
+
+
 	/* If we have a color scheme file, we switch to it, GIO will merge it in the
 	 * remmina.pref file */
 	if (g_file_test(remmina_colors_file, G_FILE_TEST_IS_REGULAR)) {
@@ -668,6 +682,16 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.periodic_rmnews_last_get = 0;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_news", "periodic_rmnews_get_count", NULL))
+		remmina_pref.periodic_rmnews_get_count = g_key_file_get_int64(gkeyfile, "remmina_news", "periodic_rmnews_get_count", NULL);
+	else
+		remmina_pref.periodic_rmnews_get_count = 0;
+
+	if (g_key_file_has_key(gkeyfile, "remmina_news", "periodic_rmnews_uuid_prefix", NULL))
+		remmina_pref.periodic_rmnews_uuid_prefix = g_key_file_get_string(gkeyfile, "remmina_news", "periodic_rmnews_uuid_prefix", NULL);
+	else
+		remmina_pref.periodic_rmnews_uuid_prefix = NULL;
+
 	/* Default settings */
 	if (!g_key_file_has_key(gkeyfile, "remmina", "name", NULL)) {
 		g_key_file_set_string(gkeyfile, "remmina", "name", "");
@@ -802,9 +826,13 @@ gboolean remmina_pref_save(void)
 
 	g_key_file_set_boolean(gkeyfile, "usage_stats", "periodic_usage_stats_permitted", remmina_pref.periodic_usage_stats_permitted);
 	g_key_file_set_int64(gkeyfile, "usage_stats", "periodic_usage_stats_last_sent", remmina_pref.periodic_usage_stats_last_sent);
-	g_key_file_set_int64(gkeyfile, "remmina_news", "periodic_rmnews_last_get", remmina_pref.periodic_rmnews_last_get);
 	g_key_file_set_string(gkeyfile, "usage_stats", "periodic_usage_stats_uuid_prefix",
 			remmina_pref.periodic_usage_stats_uuid_prefix ? remmina_pref.periodic_usage_stats_uuid_prefix : "");
+	g_key_file_set_int64(gkeyfile, "remmina_news", "periodic_rmnews_last_get", remmina_pref.periodic_rmnews_last_get);
+	g_key_file_set_integer(gkeyfile, "remmina_news", "periodic_rmnews_get_count", remmina_pref.periodic_rmnews_get_count);
+	g_key_file_set_string(gkeyfile, "remmina_news", "periodic_rmnews_uuid_prefix",
+			remmina_pref.periodic_rmnews_uuid_prefix ? remmina_pref.periodic_rmnews_uuid_prefix : "");
+
 	/* Default settings */
 	g_key_file_set_string(gkeyfile, "remmina", "name", "");
 	g_key_file_set_integer(gkeyfile, "remmina", "ignore-tls-errors", 1);
