@@ -3,7 +3,7 @@
  *
  * @copyright Copyright (C) 2010-2011 Vic Lee.
  * @copyright Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo.
- * @copyright Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo.
+ * @copyright Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +46,7 @@
 #include <vte/vte.h>
 #include <locale.h>
 #include <langinfo.h>
+#include "remmina_log.h"
 #include "remmina_public.h"
 #include "remmina_plugin_manager.h"
 #include "remmina_ssh.h"
@@ -57,11 +58,17 @@
 #define REMMINA_PLUGIN_SSH_FEATURE_TOOL_COPY  1
 #define REMMINA_PLUGIN_SSH_FEATURE_TOOL_PASTE 2
 #define REMMINA_PLUGIN_SSH_FEATURE_TOOL_SELECT_ALL 3
+#define REMMINA_PLUGIN_SSH_FEATURE_TOOL_INCREASE_FONT 4
+#define REMMINA_PLUGIN_SSH_FEATURE_TOOL_DECREASE_FONT 5
 
 #define GET_PLUGIN_DATA(gp) (RemminaPluginSshData *)g_object_get_data(G_OBJECT(gp), "plugin-data");
 
 /** Palette colors taken from sakura */
 #define PALETTE_SIZE 16
+/* Min fontsize and increase */
+#define FONT_SCALE    0.75
+#define SCALE_FACTOR  0.1
+#define FONT_MINIMAL_SIZE (PANGO_SCALE*6)
 
 enum color_schemes { LINUX, TANGO, GRUVBOX, SOLARIZED_DARK, SOLARIZED_LIGHT, XTERM, CUSTOM };
 
@@ -211,7 +218,7 @@ remmina_plugin_ssh_on_size_allocate(GtkWidget *widget, GtkAllocation *alloc, Rem
 
 
 /**
- * Remmina Protocol plugin main function.
+ * Remmina protocol plugin main function.
  *
  * First it starts the SSH tunnel if needed and then the SSH connection.
  *
@@ -226,109 +233,71 @@ remmina_plugin_ssh_main_thread(gpointer data)
 	RemminaSSH *ssh;
 	RemminaSSHShell *shell = NULL;
 	gboolean cont = FALSE;
-	gint ret;
-	const gchar *saveserver;
-	const gchar *saveusername;
 	gchar *hostport;
-	gchar tunneluser[33];           /**< On linux a username can have a 32 char lenght */
-	gchar tunnelserver[256];        /**< On linux a servername can have a 255 char lenght */
-	gchar tunnelport[6];            /**< A TCP port can have a maximum value of 65535 */
-	gchar *host;
-	gint port;
+	gint ret;
 
 	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 	CANCEL_ASYNC
 
-		gpdata = GET_PLUGIN_DATA(gp);
-
-	/**
-	 * remmina_plugin_service->protocol_plugin_start_direct_tunnel start the
-	 * SSH Tunnel and return the server + port string
-	 * Therefore we set the SSH Tunnel username here, before the tunnel
-	 * is established and than set it back to the destination SSH user.
-	 *
-	 **/
+	gpdata = GET_PLUGIN_DATA(gp);
 	remminafile = remmina_plugin_service->protocol_plugin_get_file(gp);
-	/* We save the SSH server name, so that we can restore it at the end of the connection */
-	saveserver = remmina_plugin_service->file_get_string(remminafile, "ssh_server");
-	remmina_plugin_service->file_set_string(remminafile, "save_ssh_server", g_strdup(saveserver));
-	/* We save the SSH username, so that we can restore it at the end of the connection */
-	saveusername = remmina_plugin_service->file_get_string(remminafile, "ssh_username");
-	remmina_plugin_service->file_set_string(remminafile, "save_ssh_username", g_strdup(saveusername));
-
-	if (saveserver) {
-		/** if the server string contains the character @ we extract the user
-		 * and the server string (host:port)
-		 **/
-		if (strchr(saveserver, '@')) {
-			sscanf(saveserver, "%[_a-zA-Z0-9.]@%[_a-zA-Z0-9.]:%[0-9]",
-			       tunneluser, tunnelserver, tunnelport);
-			g_debug("Username: %s, tunneluser: %s\n",
-				remmina_plugin_service->file_get_string(remminafile, "ssh_username"), tunneluser);
-			if (saveusername != NULL && tunneluser[0] != '\0')
-				remmina_plugin_service->file_set_string(remminafile, "ssh_username", NULL);
-			remmina_plugin_service->file_set_string(remminafile, "ssh_username",
-								g_strdup(tunneluser));
-			remmina_plugin_service->file_set_string(remminafile, "ssh_server",
-								g_strconcat(tunnelserver, ":", tunnelport, NULL));
-		}
-	}
 
+	/* we may need to open a new tunnel */
 	hostport = remmina_plugin_service->protocol_plugin_start_direct_tunnel(gp, 22, FALSE);
-	/* We restore the SSH username as the tunnel is set */
-	remmina_plugin_service->file_set_string(remminafile, "ssh_username", g_strdup(saveusername));
-	if (hostport == NULL)
-		return FALSE;
-	remmina_plugin_service->get_server_port(hostport, 22, &host, &port);
+	if (hostport == NULL) {
+		remmina_plugin_service->protocol_plugin_signal_connection_closed(gp);
+		return NULL;
+	}
 
 	ssh = g_object_get_data(G_OBJECT(gp), "user-data");
 	if (ssh) {
-		/* Create SSH Shell connection based on existing SSH session */
+		/* Create SSH shell connection based on existing SSH session */
+
 		shell = remmina_ssh_shell_new_from_ssh(ssh);
+		remmina_plugin_service->get_server_port(hostport, 22, &ssh->tunnel_entrance_host, &ssh->tunnel_entrance_port);
+
 		if (remmina_ssh_init_session(REMMINA_SSH(shell)) &&
-		    remmina_ssh_auth(REMMINA_SSH(shell), NULL, gp, remminafile) > 0 &&
+		    remmina_ssh_auth(REMMINA_SSH(shell), NULL, gp, remminafile) == REMMINA_SSH_AUTH_SUCCESS &&
 		    remmina_ssh_shell_open(shell, (RemminaSSHExitFunc)
-					   remmina_plugin_service->protocol_plugin_close_connection, gp))
+					   remmina_plugin_service->protocol_plugin_signal_connection_closed, gp))
 			cont = TRUE;
 	} else {
-		/* New SSH Shell connection */
-		if (remmina_plugin_service->file_get_int(remminafile, "ssh_enabled", FALSE))
-			remmina_plugin_service->file_set_string(remminafile, "ssh_server", g_strdup(hostport));
-		else
-			remmina_plugin_service->file_set_string(remminafile, "ssh_server",
-								remmina_plugin_service->file_get_string(remminafile, "server"));
-		g_free(hostport);
-		g_free(host);
 
+		/* New SSH Shell connection */
 		shell = remmina_ssh_shell_new_from_file(remminafile);
+		ssh = REMMINA_SSH(shell);
+		remmina_plugin_service->get_server_port(hostport, 22, &ssh->tunnel_entrance_host, &ssh->tunnel_entrance_port);
+
 		while (1) {
-			if (!remmina_ssh_init_session(REMMINA_SSH(shell))) {
-				remmina_plugin_service->protocol_plugin_set_error(gp, "%s", REMMINA_SSH(shell)->error);
+			if (!remmina_ssh_init_session(ssh)) {
+				REMMINA_DEBUG("init session error: %s", ssh->error);
+				remmina_plugin_service->protocol_plugin_set_error(gp, "%s", ssh->error);
 				break;
 			}
 
-			ret = remmina_ssh_auth_gui(REMMINA_SSH(shell), gp, remminafile);
-			if (ret == 0)
-				remmina_plugin_service->protocol_plugin_set_error(gp, "%s", REMMINA_SSH(shell)->error);
-			if (ret <= 0) break;
+			ret = remmina_ssh_auth_gui(ssh, gp, remminafile);
+			if (ret != REMMINA_SSH_AUTH_SUCCESS) {
+				if (ret != REMMINA_SSH_AUTH_USERCANCEL)
+					remmina_plugin_service->protocol_plugin_set_error(gp, "%s", ssh->error);
+				break;
+			}
 
 			if (!remmina_ssh_shell_open(shell, (RemminaSSHExitFunc)
-						    remmina_plugin_service->protocol_plugin_close_connection, gp)) {
-				remmina_plugin_service->protocol_plugin_set_error(gp, "%s", REMMINA_SSH(shell)->error);
+						    remmina_plugin_service->protocol_plugin_signal_connection_closed, gp)) {
+				remmina_plugin_service->protocol_plugin_set_error(gp, "%s", ssh->error);
 				break;
 			}
 
 			cont = TRUE;
 			break;
 		}
-
-		/* We restore the ssh_server name */
-		remmina_plugin_service->file_set_string(remminafile, "ssh_server",
-							remmina_plugin_service->file_get_string(remminafile, "save_ssh_server"));
 	}
+
+	g_free(hostport);
+
 	if (!cont) {
 		if (shell) remmina_ssh_shell_free(shell);
-		IDLE_ADD((GSourceFunc)remmina_plugin_service->protocol_plugin_close_connection, gp);
+		remmina_plugin_service->protocol_plugin_signal_connection_closed(gp);
 		return NULL;
 	}
 
@@ -337,10 +306,10 @@ remmina_plugin_ssh_main_thread(gpointer data)
 	gchar *charset = REMMINA_SSH(shell)->charset;
 	remmina_plugin_ssh_vte_terminal_set_encoding_and_pty(VTE_TERMINAL(gpdata->vte), charset, shell->master, shell->slave);
 
-	/* ToDo: the following call should be moved on the main thread, or something weird could happen */
+	/* TODO: The following call should be moved on the main thread, or something weird could happen */
 	remmina_plugin_ssh_on_size_allocate(GTK_WIDGET(gpdata->vte), NULL, gp);
 
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "connect");
+	remmina_plugin_service->protocol_plugin_signal_connection_opened(gp);
 
 	gpdata->thread = 0;
 	return NULL;
@@ -376,7 +345,7 @@ void remmina_plugin_ssh_vte_terminal_set_encoding_and_pty(VteTerminal *terminal,
 	vte_terminal_set_delete_binding(terminal, VTE_ERASE_DELETE_SEQUENCE);
 
 #if VTE_CHECK_VERSION(0, 38, 0)
-	/* vte_pty_new_foreig expect master FD, see https://bugzilla.gnome.org/show_bug.cgi?id=765382 */
+	/* vte_pty_new_foreign expect master FD, see https://bugzilla.gnome.org/show_bug.cgi?id=765382 */
 	vte_terminal_set_pty(terminal, vte_pty_new_foreign_sync(master, NULL, NULL));
 #else
 	vte_terminal_set_pty(terminal, master);
@@ -444,6 +413,20 @@ remmina_plugin_ssh_vte_select_all(GtkMenuItem *menuitem, gpointer vte)
 	/** @todo we should add the vte_terminal_unselect_all as well */
 }
 
+void
+remmina_plugin_ssh_vte_decrease_font(GtkMenuItem *menuitem, gpointer vte)
+{
+	TRACE_CALL(__func__);
+	vte_terminal_set_font_scale(VTE_TERMINAL(vte), vte_terminal_get_font_scale(VTE_TERMINAL(vte))-SCALE_FACTOR);
+}
+
+void
+remmina_plugin_ssh_vte_increase_font(GtkMenuItem *menuitem, gpointer vte)
+{
+	TRACE_CALL(__func__);
+	vte_terminal_set_font_scale(VTE_TERMINAL(vte), vte_terminal_get_font_scale(VTE_TERMINAL(vte))+SCALE_FACTOR);
+}
+
 void
 remmina_plugin_ssh_vte_copy_clipboard(GtkMenuItem *menuitem, gpointer vte)
 {
@@ -474,8 +457,9 @@ remmina_plugin_ssh_vte_save_session(GtkMenuItem *menuitem, RemminaProtocolWidget
 	GFileOutputStream *stream = g_file_replace(gpdata->vte_session_file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &err);
 
 	if (err != NULL) {
+		// TRANSLATORS: %s is a placeholder for an error message
 		widget = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-						_("%s"), err->message);
+						_("Error: %s"), err->message);
 		g_signal_connect(G_OBJECT(widget), "response", G_CALLBACK(gtk_widget_destroy), NULL);
 		gtk_widget_show(widget);
 		return;
@@ -492,7 +476,7 @@ remmina_plugin_ssh_vte_save_session(GtkMenuItem *menuitem, RemminaProtocolWidget
 
 	if (err == NULL) {
 		remmina_public_send_notification("remmina-terminal-saved",
-						 _("Terminal content saved under"),
+						 _("Terminal content saved in"),
 						 g_file_get_path(gpdata->vte_session_file));
 	}
 
@@ -546,15 +530,19 @@ void remmina_plugin_ssh_popup_ui(RemminaProtocolWidget *gp)
 	/* Context menu for slection and clipboard */
 	GtkWidget *menu = gtk_menu_new();
 
-	GtkWidget *select_all = gtk_menu_item_new_with_label(_("Select All (Host+a)"));
-	GtkWidget *copy = gtk_menu_item_new_with_label(_("Copy (Host+c)"));
-	GtkWidget *paste = gtk_menu_item_new_with_label(_("Paste (Host+v)"));
+	GtkWidget *select_all = gtk_menu_item_new_with_label(_("Select All (Host + A)"));
+	GtkWidget *copy = gtk_menu_item_new_with_label(_("Copy (host + C)"));
+	GtkWidget *paste = gtk_menu_item_new_with_label(_("Paste (host + V)"));
 	GtkWidget *save = gtk_menu_item_new_with_label(_("Save session to file"));
+	GtkWidget *font_incr = gtk_menu_item_new_with_label(_("Increase font size (Host + Page_Up)"));
+	GtkWidget *font_decr = gtk_menu_item_new_with_label(_("Decrease font size (Host + Page_Down)"));
 
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), select_all);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), copy);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), paste);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), save);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), font_incr);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), font_decr);
 
 	g_signal_connect(G_OBJECT(gpdata->vte), "button_press_event",
 			 G_CALLBACK(remmina_ssh_plugin_popup_menu), menu);
@@ -567,6 +555,10 @@ void remmina_plugin_ssh_popup_ui(RemminaProtocolWidget *gp)
 			 G_CALLBACK(remmina_plugin_ssh_vte_paste_clipboard), gpdata->vte);
 	g_signal_connect(G_OBJECT(save), "activate",
 			 G_CALLBACK(remmina_plugin_ssh_vte_save_session), gp);
+	g_signal_connect(G_OBJECT(font_incr), "activate",
+			 G_CALLBACK(remmina_plugin_ssh_vte_increase_font), gpdata->vte);
+	g_signal_connect(G_OBJECT(font_decr), "activate",
+			 G_CALLBACK(remmina_plugin_ssh_vte_decrease_font), gpdata->vte);
 
 	gtk_widget_show_all(menu);
 }
@@ -647,7 +639,7 @@ remmina_plugin_ssh_init(RemminaProtocolWidget *gp)
 		for (i = 0; dirs[i] != NULL; ++i) {
 			remmina_dir = g_build_path("/", dirs[i], "remmina", "theme", NULL);
 			GDir *system_data_dir = g_dir_open(remmina_dir, 0, &error);
-			// ignoring this error is ok, because the folder may not existing
+			// ignoring this error is OK, because the folder may not exist
 			if (error) {
 				g_error_free(error);
 				error = NULL;
@@ -828,6 +820,8 @@ remmina_plugin_ssh_init(RemminaProtocolWidget *gp)
 	else
 		sshlogname = remmina_plugin_service->file_get_string(remminafile, "sshlogname");
 
+	sshlogname = remmina_file_format_properties(remminafile, sshlogname);
+
 	fp = g_strconcat(dir, "/", sshlogname, NULL);
 	gpdata->vte_session_file = g_file_new_for_path(fp);
 
@@ -835,7 +829,7 @@ remmina_plugin_ssh_init(RemminaProtocolWidget *gp)
 }
 
 /**
- * Initialize the the main window properties and the pthread.
+ * Initialize the main window properties and the pthread.
  *
  * The call of this function is a requirement of remmina_protocol_widget_open_connection_real().
  * @return TRUE
@@ -883,12 +877,12 @@ remmina_plugin_ssh_close_connection(RemminaProtocolWidget *gp)
 		gpdata->shell = NULL;
 	}
 
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "disconnect");
+	remmina_plugin_service->protocol_plugin_signal_connection_closed(gp);
 	return FALSE;
 }
 
 /**
- * Not used by the the plugin.
+ * Not used by the plugin.
  *
  * @return Always TRUE
  */
@@ -946,6 +940,14 @@ remmina_plugin_ssh_call_feature(RemminaProtocolWidget *gp, const RemminaProtocol
 	case REMMINA_PLUGIN_SSH_FEATURE_TOOL_SELECT_ALL:
 		vte_terminal_select_all(VTE_TERMINAL(gpdata->vte));
 		return;
+	case REMMINA_PLUGIN_SSH_FEATURE_TOOL_INCREASE_FONT:
+		vte_terminal_set_font_scale(VTE_TERMINAL(gpdata->vte),
+				vte_terminal_get_font_scale(VTE_TERMINAL(gpdata->vte))+SCALE_FACTOR);
+		return;
+	case REMMINA_PLUGIN_SSH_FEATURE_TOOL_DECREASE_FONT:
+		vte_terminal_set_font_scale(VTE_TERMINAL(gpdata->vte),
+				vte_terminal_get_font_scale(VTE_TERMINAL(gpdata->vte))-SCALE_FACTOR);
+		return;
 	}
 }
 
@@ -1034,6 +1036,8 @@ static RemminaProtocolFeature remmina_plugin_ssh_features[] =
 	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL, REMMINA_PLUGIN_SSH_FEATURE_TOOL_COPY,	  N_("Copy"),	    N_("_Copy"),       NULL },
 	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL, REMMINA_PLUGIN_SSH_FEATURE_TOOL_PASTE,	  N_("Paste"),	    N_("_Paste"),      NULL },
 	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL, REMMINA_PLUGIN_SSH_FEATURE_TOOL_SELECT_ALL, N_("Select all"), N_("_Select all"), NULL },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL, REMMINA_PLUGIN_SSH_FEATURE_TOOL_INCREASE_FONT, N_("Increase font size"), N_("_Increase font size"), NULL },
+	{ REMMINA_PROTOCOL_FEATURE_TYPE_TOOL, REMMINA_PLUGIN_SSH_FEATURE_TOOL_DECREASE_FONT, N_("Decrease font size"), N_("_Decrease font size"), NULL },
 	{ REMMINA_PROTOCOL_FEATURE_TYPE_END,  0,					  NULL,		    NULL,	       NULL }
 };
 
@@ -1045,21 +1049,31 @@ static RemminaProtocolFeature remmina_plugin_ssh_features[] =
  *  3. Setting description.
  *  4. Compact disposition.
  *  5. Values for REMMINA_PROTOCOL_SETTING_TYPE_SELECT or REMMINA_PROTOCOL_SETTING_TYPE_COMBO.
- *  6. Unused pointer.
+ *  6. Setting Tooltip.
  * .
  */
 static const RemminaProtocolSetting remmina_ssh_basic_settings[] =
 {
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SERVER,	  "ssh_server",	    NULL,			  FALSE, "_ssh._tcp", NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "ssh_username",   N_("User name"),		  FALSE, NULL,	      NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "ssh_password",   N_("User password"),	  FALSE, NULL,	      NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "ssh_auth",	    N_("Authentication type"),	  FALSE, ssh_auth,    NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_FILE,	  "ssh_privatekey", N_("Identity file"),	  FALSE, NULL,	      NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "ssh_passphrase", N_("Private key passphrase"), FALSE, NULL,	      NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "exec",	    N_("Startup program"),	  FALSE, NULL,	      NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,		    NULL,			  FALSE, NULL,	      NULL }
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SERVER,	  "server",	    NULL,				  FALSE, "_ssh._tcp", NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "username",	    N_("Username"),			  FALSE, NULL,	      NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "password",	    N_("User password"),		  FALSE, NULL,	      NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT,	  "ssh_auth",	    N_("Authentication type"),		  FALSE, ssh_auth,    NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_FILE,	  "ssh_privatekey", N_("Identity file"),		  FALSE, NULL,	      NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, "ssh_passphrase", N_("Password to unlock private key"), FALSE, NULL,	      NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	  "exec",	    N_("Startup program"),		  FALSE, NULL,	      NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	  NULL,		    NULL,				  FALSE, NULL,	      NULL }
 };
 
+static gchar log_tips[] =
+        N_("The filename can use the following placeholders:\n\n"
+        "  • %h is substituted with the server name\n"
+        "  • %t is substituted with the SSH server name\n"
+        "  • %u is substituted with the username\n"
+        "  • %U is substituted with the SSH username\n"
+        "  • %p is substituted with Remmina profile name\n"
+        "  • %g is substituted with Remmina profile group name\n"
+        "  • %d is substituted with local datetime in iso8601 format\n");
+
 /**
  * Array of RemminaProtocolSetting for advanced settings.
  * - Each item is composed by:
@@ -1068,25 +1082,25 @@ static const RemminaProtocolSetting remmina_ssh_basic_settings[] =
  *  3. Setting description.
  *  4. Compact disposition.
  *  5. Values for REMMINA_PROTOCOL_SETTING_TYPE_SELECT or REMMINA_PROTOCOL_SETTING_TYPE_COMBO.
- *  6. Unused pointer.
+ *  6. Setting Tooltip.
  *
  */
 static const RemminaProtocolSetting remmina_ssh_advanced_settings[] =
 {
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "ssh_color_scheme",	  N_("Terminal color scheme"),		    FALSE, ssh_terminal_palette, NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "ssh_charset",		  N_("Character set"),			    FALSE, ssh_charset_list,	 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	"ssh_proxycommand",	  N_("SSH Proxy Command"),		    FALSE, NULL,		 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	"ssh_kex_algorithms",	  N_("KEX (Key Exchange) algorithms"),	    FALSE, NULL,		 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	"ssh_ciphers",		  N_("Symmetric cipher client to server"),  FALSE, NULL,		 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	"ssh_hostkeytypes",	  N_("Preferred server host key types"),    FALSE, NULL,		 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_FOLDER, "sshlogfolder",		  N_("SSH session log folder"),		    FALSE, NULL,		 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	"sshlogname",		  N_("SSH session log file name"),	    FALSE, NULL,		 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"sshlogenabled",	  N_("Enable SSH session logging at exit"), FALSE, NULL,		 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"audiblebell",		  N_("Enable terminal audible bell"),	    FALSE, NULL,		 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"ssh_compression",	  N_("Enable SSH compression"),		    FALSE, NULL,		 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"disablepasswordstoring", N_("Disable password storing"),	    TRUE,  NULL,		 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"ssh_stricthostkeycheck", N_("Strict host key checking"),	    TRUE,  NULL,		 NULL },
-	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	NULL,			  NULL,					    FALSE, NULL,		 NULL }
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "ssh_color_scheme",	  N_("Terminal color scheme"),		      FALSE, ssh_terminal_palette, NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "ssh_charset",		  N_("Character set"),			      FALSE, ssh_charset_list,	   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	"ssh_proxycommand",	  N_("SSH Proxy Command"),		      FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	"ssh_kex_algorithms",	  N_("KEX (Key Exchange) algorithms"),	      FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	"ssh_ciphers",		  N_("Symmetric cipher client to server"),    FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	"ssh_hostkeytypes",	  N_("Preferred server host key types"),      FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_FOLDER, "sshlogfolder",		  N_("Folder for SSH session log"),	      FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_TEXT,	"sshlogname",		  N_("Filename for SSH session log"),	      FALSE, NULL,		   log_tips },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"sshlogenabled",	  N_("Log SSH session when exiting Remmina"), FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"audiblebell",		  N_("Audible terminal bell"),		      FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"ssh_compression",	  N_("SSH compression"),		      FALSE, NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"disablepasswordstoring", N_("Don't remember passwords"),	      TRUE,  NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_CHECK,	"ssh_stricthostkeycheck", N_("Strict host key checking"),	      TRUE,  NULL,		   NULL },
+	{ REMMINA_PROTOCOL_SETTING_TYPE_END,	NULL,			  NULL,					      FALSE, NULL,		   NULL }
 };
 
 /**
@@ -1121,7 +1135,7 @@ static RemminaProtocolPlugin remmina_plugin_ssh =
 /*
  * this function is used for
  * - inserting into the list to became a sorted list [g_list_insert_sorted()]
- * - checking the list to avoid doublicate entries [g_list_find_custom()]
+ * - checking the list to avoid duplicate entries [g_list_find_custom()]
  */
 static gint
 compare(gconstpointer a, gconstpointer b)
@@ -1138,7 +1152,7 @@ remmina_ssh_plugin_load_terminal_palettes(gpointer *ssh_terminal_palette_new)
 	GList *files = NULL;
 	unsigned int rec_size = 0;
 	/*
-	 * count number of (all) files to reserve enought memory
+	 * count number of (all) files to reserve enough memory
 	 */
 	/* /usr/local/share/remmina */
 	const gchar *const *dirs = g_get_system_data_dirs();
@@ -1150,7 +1164,7 @@ remmina_ssh_plugin_load_terminal_palettes(gpointer *ssh_terminal_palette_new)
 		gchar *remmina_dir = g_build_path("/", dirs[i], "remmina", "theme", NULL);
 		system_data_dir = g_dir_open(remmina_dir, 0, &error);
 		g_free(remmina_dir);
-		// ignoring this error is ok, because the folder may not existing
+		// ignoring this error is OK, because the folder may not exist
 		if (error) {
 			g_error_free(error);
 			error = NULL;
@@ -1229,7 +1243,8 @@ remmina_ssh_plugin_register(void)
 	remmina_plugin_ssh_features[0].opt3 = GUINT_TO_POINTER(remmina_pref.vte_shortcutkey_copy);
 	remmina_plugin_ssh_features[1].opt3 = GUINT_TO_POINTER(remmina_pref.vte_shortcutkey_paste);
 	remmina_plugin_ssh_features[2].opt3 = GUINT_TO_POINTER(remmina_pref.vte_shortcutkey_select_all);
-	remmina_plugin_ssh_features[3].opt3 = GUINT_TO_POINTER(remmina_pref.vte_shortcutkey_select_all);
+	remmina_plugin_ssh_features[3].opt3 = GUINT_TO_POINTER(remmina_pref.vte_shortcutkey_increase_font);
+	remmina_plugin_ssh_features[4].opt3 = GUINT_TO_POINTER(remmina_pref.vte_shortcutkey_decrease_font);
 
 	remmina_plugin_service = &remmina_plugin_manager_service;
 
